@@ -36,10 +36,12 @@ static struct { const char *name; int algo;} pubkey_names[] = {
     { "DSA",           PUBKEY_ALGO_DSA     },
     {NULL} };
 
+/* Note: the first string is the one used by ascii armor */
 static struct { const char *name; int algo;} digest_names[] = {
     { "MD5",           DIGEST_ALGO_MD5    },
     { "SHA1",          DIGEST_ALGO_SHA1   },
     { "SHA-1",         DIGEST_ALGO_SHA1   },
+    { "RIPEMD160",     DIGEST_ALGO_RMD160 },
     { "RMD160",        DIGEST_ALGO_RMD160 },
     { "RMD-160",       DIGEST_ALGO_RMD160 },
     { "RIPE-MD-160",   DIGEST_ALGO_RMD160 },
@@ -134,6 +136,7 @@ check_pubkey_algo2( int algo, unsigned usage )
 	return 0;
 
       case PUBKEY_ALGO_ELGAMAL:
+      case PUBKEY_ALGO_ELGAMAL_E:
 	return 0;
 
     #ifdef HAVE_RSA_CIPHER
