@@ -28,13 +28,14 @@
 #include "packet.h"
 #include "mpi.h"
 #include "cipher.h"
+#include "options.h"
 
 
 static int decode_filter( void *opaque, int control, IOBUF a,
 					byte *buf, size_t *ret_len);
 
 typedef struct {
-    BLOWFISH_context *bf_ctx;
+    CIPHER_HANDLE cipher_hd;
 } decode_filter_ctx_t;
 
 
@@ -47,19 +48,24 @@ decrypt_data( PKT_encrypted *ed, DEK *dek )
 {
     decode_filter_ctx_t dfx;
     byte *p;
-    int c, i;
+    int rc, c, i;
     byte temp[16];
 
-
-    if( dek->algo != CIPHER_ALGO_BLOWFISH
-	&& dek->algo != CIPHER_ALGO_BLOWFISH128 )
-	return G10ERR_CIPHER_ALGO;
+    if( opt.verbose ) {
+	const char *s = cipher_algo_to_string( dek->algo );
+	if( s )
+	    log_info("%s encrypted data\n", s );
+	else
+	    log_info("encrypted with unknown algorithm %d\n", dek->algo );
+    }
+    if( (rc=check_cipher_algo(dek->algo)) )
+	return rc;
     if( ed->len && ed->len < 10 )
 	log_bug("Nanu\n");   /* oops: found a bug */
 
-    dfx.bf_ctx = m_alloc_secure( sizeof *dfx.bf_ctx );
-    blowfish_setkey( dfx.bf_ctx, dek->key, dek->keylen	);
-    blowfish_setiv( dfx.bf_ctx, NULL );
+    dfx.cipher_hd = cipher_open( dek->algo, CIPHER_MODE_AUTO_CFB, 1 );
+    cipher_setkey( dfx.cipher_hd, dek->key, dek->keylen  );
+    cipher_setiv( dfx.cipher_hd, NULL );
 
     if( ed->len ) {
 	iobuf_set_limit( ed->buf, ed->len );
@@ -74,10 +80,11 @@ decrypt_data( PKT_encrypted *ed, DEK *dek )
 	    else
 		temp[i] = c;
     }
-    blowfish_decode_cfb( dfx.bf_ctx, temp, temp, 10);
+    cipher_decrypt( dfx.cipher_hd, temp, temp, 10);
+    cipher_sync( dfx.cipher_hd );
     p = temp;
     if( p[6] != p[8] || p[7] != p[9] ) {
-	m_free(dfx.bf_ctx);
+	cipher_close(dfx.cipher_hd);
 	return G10ERR_BAD_KEY;
     }
     iobuf_push_filter( ed->buf, decode_filter, &dfx );
@@ -88,7 +95,7 @@ decrypt_data( PKT_encrypted *ed, DEK *dek )
     else
 	iobuf_clear_eof( ed->buf );
     ed->buf = NULL;
-    m_free(dfx.bf_ctx);
+    cipher_close(dfx.cipher_hd);
     return 0;
 }
 
@@ -109,7 +116,7 @@ decode_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len)
 	}
 
 	if( n )
-	    blowfish_decode_cfb( fc->bf_ctx, buf, buf, n);
+	    cipher_decrypt( fc->cipher_hd, buf, buf, n);
 	else
 	    rc = -1; /* eof */
 	*ret_len = n;
