@@ -25,6 +25,10 @@
 #include "cipher.h"
 #include "keydb.h"
 
+#define DEFAULT_CIPHER_ALGO  CIPHER_ALGO_BLOWFISH
+#define DEFAULT_PUBKEY_ALGO  PUBKEY_ALGO_ELGAMAL
+#define DEFAULT_DIGEST_ALGO  DIGEST_ALGO_RMD160
+
 
 typedef struct {
     int header_okay;
@@ -41,6 +45,7 @@ typedef struct {
 #endif
 
 /*-- misc.c --*/
+void trap_unaligned(void);
 u16 checksum_u16( unsigned n );
 u16 checksum( byte *p, unsigned n );
 u16 checksum_mpi( MPI a );
@@ -70,6 +75,7 @@ int change_passphrase( const char *username );
 
 /*-- keygen.c --*/
 void generate_keypair(void);
+void generate_subkeypair(const char *userid);
 
 /*-- openfile.c --*/
 int overwrite_filep( const char *fname );
